import argparse
import requests, sys

from tqdm import tqdm


DOWNLOAD_URL = (
    "https://download.nlm.nih.gov/umls/kss/2021AB/umls-2021AB-metathesaurus.zip"
)


def get_tgtt_ticket(apikey):
    url = "https://utslogin.nlm.nih.gov/cas/v1/api-key"
    param = {"apikey": apikey}
    headers = {"Content-type": "application/x-www-form-urlencoded"}

    TGTresponse = requests.post(url, headers=headers, data=param)
    _, second = TGTresponse.text.split("api-key/")
    TGTTicket, _ = second.split('" method')

    return TGTTicket


def get_url(TGTTicket):
    url = "https://utslogin.nlm.nih.gov/cas/v1/tickets/" + TGTTicket
    param = {"service": DOWNLOAD_URL}
    headers = {"Content-type": "application/x-www-form-urlencoded"}

    STResponse = requests.post(url, headers=headers, data=param)

    print(STResponse.text)

    url = DOWNLOAD_URL + "?ticket=" + STResponse.text

    return url


def _download_umls(url, output_location):
    value = DOWNLOAD_URL.split("/")
    r = requests.get(url, allow_redirects=True, stream=True)
    total_size_in_bytes = int(r.headers.get("content-length", 0))
    block_size = 1024  # 1 Kibibyte
    progress_bar = tqdm(total=total_size_in_bytes, unit="iB", unit_scale=True)
    with open(output_location + value[len(value) - 1], "wb") as f:
        for data in r.iter_content(block_size):
            progress_bar.update(len(data))
            f.write(data)
    progress_bar.close()
    return r


def download_umls(apikey, output_location):
    if not DOWNLOAD_URL:
        sys.exit("Download_url is empty")

    TGTTicket = get_tgtt_ticket(apikey)
    url = get_url(TGTTicket)
    r = _download_umls(url, output_location)

    # Retrieve HTTP meta-data
    print(r.status_code)
    print(r.headers["content-type"])
    print(r.encoding)

    print("completed")


def main():
    # TODO: Be able to specify the version of UMLS that you want.
    parser = argparse.ArgumentParser(description="process user given parameters")
    parser.add_argument(
        "-k",
        "--apikey",
        required=True,
        dest="apikey",
        help="enter api key from your UTS Profile",
    )
    parser.add_argument(
        "-v",
        "--version",
        required=False,
        dest="version",
        default="current",
        help="enter version example-2015AA",
    )
    parser.add_argument(
        "-f",
        "--output_location",
        required=True,
        dest="output_location",
        help="enter a name for your output directory",
    )
    args = parser.parse_args()
    if not args.apikey:
        import dbutils

        apikey = dbutils.secrets.get(scope="umls", key="api_key")
        sys.exit("Please enter your api key.")
    else:
        apikey = args.apikey
    if not args.output_location:
        sys.exit("Please enter an output directory.")
    download_umls(apikey, args.output_location)


# main()
